::Chef::Recipe.send(:include, CycleCloud::Mounts::Helpers)


define :lustre_mount, :data => "" do
    name = params[:name]
    mount = params[:data]
    defaults = node['cshared']['client']['lustre']['defaults']
    mount = apply_defaults(name, mount, defaults)

    Chef::Log.info "Mounting lustre filesystem #{name} at #{mount['mountpoint']} from #{mount['address']}@#{mount['export_path']} with options #{mount['options']}"
    if mount['address'].nil?
      raise "Mount #{name} has type #{mount['type']} but no 'address' provided: {mount.inspect}"
    end
    if ! lustre_mount?(mount)
        raise "Attempt to mount #{name} as lustre: {mount.inspect}"
    end

    mountpoint = mount['mountpoint']
  
    mountoptions = mount['options']
    if mountoptions.nil? || mountoptions.empty?
        mountoptions = ["noatime","flock"]
        Chef::Log.warn "No mount options provided for #{name}, using defaults #{mountoptions}"
    end
    
    # Create mount point (defaults to same path as the export on the server)
    # TODO: should probably make owner/perms configurable
    #       This is difficult since users/groups generally do not exist yet...  (maybe set uid/gid?)
    directory mountpoint  do
        owner "root"
        group "root"
        mode "0755"
        recursive true
        # Do not modify the directory once it is created (allows later recipes to change ownership)
        not_if "test -d #{mountpoint}"
    end

    mount mountpoint do
        device "#{mount['address']}@#{mount['export_path']}"
        fstype 'lustre'
        pass mount['pass']
        options mountoptions
        action [:mount, :enable]
        retries 5
        retry_delay 30
    end

    # Lustre requires unmounting before termination:
    # See: https://techcommunity.microsoft.com/t5/azure-high-performance-computing/unmounting-azure-managed-lustre-filesystem-in-a-cyclecloud-hpc/ba-p/3945202
    scripts_dir = "#{node.default['cyclecloud']['home']}/scripts"
    directory scripts_dir  do
        owner "root"
        group "cyclecloud"
        mode "0755"
        # Do not modify the directory once it is created
        not_if "test -d #{node.default['cyclecloud']['home']}/scripts"
    end

    # TODO: Scheduled events MUST be enabled in the cluster template for these handlers to be called reliably.
    # Add the following to the node/nodearray definition:
    #   EnableTerminateNotification = true

    # Register a unique unmount action for each lustre mountpoint
    unmount_script = "#{scripts_dir}/unmount_#{name}.sh"
    template unmount_script do
        source "unmount_lustre.sh.erb"
        owner "root"
        group "cyclecloud"
        mode "0750"
        variables(:mountpoint => mountpoint)
    end

    execute "#{node['cyclecloud']['jetpack']['executable']} run_on_shutdown #{unmount_script}"
      
end
  